@@ -440,7 +440,8 @@ class DeviantartExtractor(Extractor):
                     html.append("text-align:")
                     html.append(attrs["textAlign"])
                     html.append(";")
-                html.append('margin-inline-start:0px">')
+                self._tiptap_process_indentation(html, attrs)
+                html.append('">')
 
                 for block in children:
                     self._tiptap_process_content(html, block)
@@ -460,17 +461,32 @@ class DeviantartExtractor(Extractor):
             html.append(' style="text-align:')
             html.append(attrs.get("textAlign") or "left")
             html.append('">')
-            html.append('<span style="margin-inline-start:0px">')
-
-            children = content.get("content")
-            if children:
-                for block in children:
-                    self._tiptap_process_content(html, block)
-
+            html.append('<span style="')
+            self._tiptap_process_indentation(html, attrs)
+            html.append('">')
+            self._tiptap_process_children(html, content)
             html.append("</span></h")
             html.append(level)
             html.append(">")
 
+        elif type in ("listItem", "bulletList", "orderedList", "blockquote"):
+            c = type[1]
+            tag = (
+                "li" if c == "i" else
+                "ul" if c == "u" else
+                "ol" if c == "r" else
+                "blockquote"
+            )
+            html.append("<" + tag + ">")
+            self._tiptap_process_children(html, content)
+            html.append("</" + tag + ">")
+
+        elif type == "anchor":
+            attrs = content["attrs"]
+            html.append('<a id="')
+            html.append(attrs.get("id") or "")
+            html.append('" data-testid="anchor"></a>')
+
         elif type == "hardBreak":
             html.append("<br/><br/>")
 
@@ -488,6 +504,44 @@ class DeviantartExtractor(Extractor):
             html.append(user)
             html.append('</a>')
 
+        elif type == "da-gif":
+            attrs = content["attrs"]
+            width = str(attrs.get("width") or "")
+            height = str(attrs.get("height") or "")
+            url = text.escape(attrs.get("url") or "")
+
+            html.append('<div data-da-type="da-gif" data-width="')
+            html.append(width)
+            html.append('" data-height="')
+            html.append(height)
+            html.append('" data-alignment="')
+            html.append(attrs.get("alignment") or "")
+            html.append('" data-url="')
+            html.append(url)
+            html.append('" class="t61qu"><video role="img" autoPlay="" '
+                        'muted="" loop="" style="pointer-events:none" '
+                        'controlsList="nofullscreen" playsInline="" '
+                        'aria-label="gif" data-da-type="da-gif" width="')
+            html.append(width)
+            html.append('" height="')
+            html.append(height)
+            html.append('" src="')
+            html.append(url)
+            html.append('" class="_1Fkk6"></video></div>')
+
+        elif type == "da-video":
+            src = text.escape(content["attrs"].get("src") or "")
+            html.append('<div data-testid="video" data-da-type="da-video" '
+                        'data-src="')
+            html.append(src)
+            html.append('" class="_1Uxvs"><div data-canfs="yes" data-testid="v'
+                        'ideo-inner" class="main-video" style="width:780px;hei'
+                        'ght:438px"><div style="width:780px;height:438px">'
+                        '<video src="')
+            html.append(src)
+            html.append('" style="width:100%;height:100%;" preload="auto" cont'
+                        'rols=""></video></div></div></div>')
+
         else:
             self.log.warning("Unsupported content type '%s'", type)
 
@@ -501,7 +555,13 @@ class DeviantartExtractor(Extractor):
                     attrs = mark.get("attrs") or {}
                     html.append('<a href="')
                     html.append(text.escape(attrs.get("href") or ""))
-                    html.append('" rel="noopener noreferrer nofollow ugc">')
+                    if "target" in attrs:
+                        html.append('" target="')
+                        html.append(attrs["target"])
+                    html.append('" rel="')
+                    html.append(attrs.get("rel") or
+                                "noopener noreferrer nofollow ugc")
+                    html.append('">')
                     close.append("</a>")
                 elif type == "bold":
                     html.append("<strong>")
@@ -525,6 +585,18 @@ class DeviantartExtractor(Extractor):
         else:
             html.append(text.escape(content["text"]))
 
+    def _tiptap_process_children(self, html, content):
+        children = content.get("content")
+        if children:
+            for block in children:
+                self._tiptap_process_content(html, block)
+
+    def _tiptap_process_indentation(self, html, attrs):
+        itype = ("text-indent" if attrs.get("indentType") == "line" else
+                 "margin-inline-start")
+        isize = str((attrs.get("indentation") or 0) * 24)
+        html.append(itype + ":" + isize + "px")
+
     def _tiptap_process_deviation(self, html, content):
         dev = content["attrs"]["deviation"]
         media = dev.get("media") or ()
@@ -615,10 +687,18 @@ x2="45.4107524%" y2="71.4898596%" id="app-root-3">\
             for folder in folders:
                 if match(folder["name"]):
                     return folder
+                elif folder.get("has_subfolders"):
+                    for subfolder in folder["subfolders"]:
+                        if match(subfolder["name"]):
+                            return subfolder
         else:
             for folder in folders:
                 if folder["folderid"] == uuid:
                     return folder
+                elif folder.get("has_subfolders"):
+                    for subfolder in folder["subfolders"]:
+                        if subfolder["folderid"] == uuid:
+                            return subfolder
         raise exception.NotFoundError("folder")
 
     def _folder_urls(self, folders, category, extractor):
@@ -750,7 +830,7 @@ x2="45.4107524%" y2="71.4898596%" id="app-root-3">\
                 username, folder["gallery_id"], public=False):
             cache[dev["deviationid"]] = dev if has_access else None
 
-        return cache[deviation["deviationid"]]
+        return cache.get(deviation["deviationid"])
 
     def _unwatch_premium(self):
         for username in self.unwatch:
@@ -758,19 +838,22 @@ x2="45.4107524%" y2="71.4898596%" id="app-root-3">\
             self.api.user_friends_unwatch(username)
 
     def _eclipse_media(self, media, format="preview"):
-        url = [media["baseUri"], ]
+        url = [media["baseUri"]]
 
         formats = {
             fmt["t"]: fmt
             for fmt in media["types"]
         }
 
-        tokens = media["token"]
-        if len(tokens) == 1:
-            fmt = formats[format]
-            url.append(fmt["c"].replace("<prettyName>", media["prettyName"]))
-        url.append("?token=")
-        url.append(tokens[-1])
+        tokens = media.get("token") or ()
+        if tokens:
+            if len(tokens) <= 1:
+                fmt = formats[format]
+                if "c" in fmt:
+                    url.append(fmt["c"].replace(
+                        "<prettyName>", media["prettyName"]))
+            url.append("?token=")
+            url.append(tokens[-1])
 
         return "".join(url), formats
 
@@ -816,7 +899,8 @@ class DeviantartGalleryExtractor(DeviantartExtractor):
     """Extractor for all deviations from an artist's gallery"""
     subcategory = "gallery"
     archive_fmt = "g_{_username}_{index}.{extension}"
-    pattern = BASE_PATTERN + r"/gallery(?:/all|/?\?catpath=)?/?$"
+    pattern = (BASE_PATTERN + r"/gallery"
+               r"(?:/all|/recommended-for-you|/?\?catpath=)?/?$")
     example = "https://www.deviantart.com/USER/gallery/"
 
     def deviations(self):
@@ -912,13 +996,36 @@ class DeviantartFolderExtractor(DeviantartExtractor):
     def deviations(self):
         folders = self.api.gallery_folders(self.user)
         folder = self._find_folder(folders, self.folder_name, self.folder_id)
+
+        # Leaving this here for backwards compatibility
         self.folder = {
             "title": folder["name"],
             "uuid" : folder["folderid"],
             "index": self.folder_id,
             "owner": self.user,
+            "parent_uuid": folder["parent"],
         }
-        return self.api.gallery(self.user, folder["folderid"], self.offset)
+
+        if folder.get("subfolder"):
+            self.folder["parent_folder"] = folder["parent_folder"]
+            self.archive_fmt = "F_{folder[parent_uuid]}_{index}.{extension}"
+
+            if self.flat:
+                self.directory_fmt = ("{category}", "{username}",
+                                      "{folder[parent_folder]}")
+            else:
+                self.directory_fmt = ("{category}", "{username}",
+                                      "{folder[parent_folder]}",
+                                      "{folder[title]}")
+
+        if folder.get("has_subfolders") and self.config("subfolders", True):
+            for subfolder in folder["subfolders"]:
+                subfolder["parent_folder"] = folder["name"]
+                subfolder["subfolder"] = True
+            yield from self._folder_urls(
+                folder["subfolders"], "gallery", DeviantartFolderExtractor)
+
+        yield from self.api.gallery(self.user, folder["folderid"], self.offset)
 
     def prepare(self, deviation):
         DeviantartExtractor.prepare(self, deviation)
@@ -929,7 +1036,7 @@ class DeviantartStashExtractor(DeviantartExtractor):
     """Extractor for sta.sh-ed deviations"""
     subcategory = "stash"
     archive_fmt = "{index}.{extension}"
-    pattern = (r"(?:https?://)?(?:(?:www\.)?deviantart\.com/stash|sta\.sh)"
+    pattern = (r"(?:https?://)?(?:(?:www\.)?deviantart\.com/stash|sta\.s(h))"
                r"/([a-z0-9]+)")
     example = "https://www.deviantart.com/stash/abcde"
 
@@ -941,9 +1048,18 @@ class DeviantartStashExtractor(DeviantartExtractor):
 
     def deviations(self, stash_id=None):
         if stash_id is None:
-            stash_id = self.groups[0]
-        url = "https://www.deviantart.com/stash/" + stash_id
-        page = self._limited_request(url).text
+            legacy_url, stash_id = self.groups
+        else:
+            legacy_url = False
+
+        if legacy_url and stash_id[0] == "2":
+            url = "https://sta.sh/" + stash_id
+            response = self._limited_request(url)
+            stash_id = response.url.rpartition("/")[2]
+            page = response.text
+        else:
+            url = "https://www.deviantart.com/stash/" + stash_id
+            page = self._limited_request(url).text
 
         if stash_id[0] == "0":
             uuid = text.extr(page, '//deviation/', '"')
@@ -1160,7 +1276,34 @@ class DeviantartDeviationExtractor(DeviantartExtractor):
 
         deviation = self.api.deviation(uuid)
         deviation["_page"] = page
-        return (deviation,)
+
+        _dev_info = text.extr(
+            page, '\\"deviationExtended\\":', ',\\"deviation\\":', None)
+        # Clean up escaped quotes
+        _json_str = re.sub(
+            r'(?<!\\)\\{1}"', '"', _dev_info).replace("\\'", "'")
+        _extended_info = util.json_loads(_json_str)[self.deviation_id]
+        additional_media = _extended_info.get("additionalMedia") or ()
+
+        if additional_media:
+            self.filename_fmt = ("{category}_{index}_{index_file}_{title}_"
+                                 "{num:>02}.{extension}")
+            self.archive_fmt = ("g_{_username}_{index}{index_file:?_//}."
+                                "{extension}")
+
+        deviation["index_file"] = 0
+        deviation["count"] = 1 + len(additional_media)
+        deviation["num"] = 1
+        yield deviation
+
+        for index, post in enumerate(additional_media):
+            uri = post["media"]["baseUri"].encode().decode("unicode-escape")
+            deviation["content"]["src"] = uri
+            deviation["num"] += 1
+            deviation["index_file"] = post["fileId"]
+            # Download only works on purchased materials - no way to check
+            deviation["is_downloadable"] = False
+            yield deviation
 
 
 class DeviantartScrapsExtractor(DeviantartExtractor):
@@ -1291,7 +1434,7 @@ class DeviantartOAuthAPI():
     def __init__(self, extractor):
         self.extractor = extractor
         self.log = extractor.log
-        self.headers = {"dA-minor-version": "20200519"}
+        self.headers = {"dA-minor-version": "20210526"}
         self._warn_429 = True
 
         self.delay = extractor.config("wait-min", 0)
