@@ -23,7 +23,7 @@ class GelbooruBase():
     root = "https://gelbooru.com"
     offset = 0
 
-    def _api_request(self, params, key="post"):
+    def _api_request(self, params, key="post", log=False):
         if "s" not in params:
             params["s"] = "post"
         params["api_key"] = self.api_key
@@ -32,11 +32,15 @@ class GelbooruBase():
         url = self.root + "/index.php?page=dapi&q=index&json=1"
         data = self.request(url, params=params).json()
 
+        if not key:
+            return data
+
         try:
             posts = data[key]
         except KeyError:
-            self.log.error("Incomplete API response (missing '%s')", key)
-            self.log.debug("%s", data)
+            if log:
+                self.log.error("Incomplete API response (missing '%s')", key)
+                self.log.debug("%s", data)
             return []
 
         if not isinstance(posts, list):
@@ -47,19 +51,44 @@ class GelbooruBase():
         params["pid"] = self.page_start
         params["limit"] = self.per_page
         limit = self.per_page // 2
+        pid = False
+
+        if "tags" in params:
+            tags = params["tags"].split()
+            op = "<"
+            id = False
+
+            for tag in tags:
+                if tag.startswith("sort:"):
+                    if tag == "sort:id:asc":
+                        op = ">"
+                    elif tag == "sort:id" or tag.startswith("sort:id:"):
+                        op = "<"
+                    else:
+                        pid = True
+                elif tag.startswith("id:"):
+                    id = True
+
+            if not pid:
+                if id:
+                    tag = "id:" + op
+                    tags = [t for t in tags if not t.startswith(tag)]
+                tags = "{} id:{}".format(" ".join(tags), op)
 
         while True:
             posts = self._api_request(params)
 
-            for post in posts:
-                yield post
+            yield from posts
 
             if len(posts) < limit:
                 return
 
-            if "pid" in params:
-                del params["pid"]
-            params["tags"] = "{} id:<{}".format(self.tags, post["id"])
+            if pid:
+                params["pid"] += 1
+            else:
+                if "pid" in params:
+                    del params["pid"]
+                params["tags"] = tags + str(posts[-1]["id"])
 
     def _pagination_html(self, params):
         url = self.root + "/index.php"
@@ -85,11 +114,12 @@ class GelbooruBase():
             md5 = post["md5"]
             path = "/images/{}/{}/{}.webm".format(md5[0:2], md5[2:4], md5)
             post["_fallback"] = GelbooruBase._video_fallback(path)
-            url = "https://img3.gelbooru.com" + path
+            url = "https://img4.gelbooru.com" + path
         return url
 
     @staticmethod
     def _video_fallback(path):
+        yield "https://img3.gelbooru.com" + path
         yield "https://img2.gelbooru.com" + path
         yield "https://img1.gelbooru.com" + path
 
@@ -117,7 +147,7 @@ class GelbooruBase():
 class GelbooruTagExtractor(GelbooruBase,
                            gelbooru_v02.GelbooruV02TagExtractor):
     """Extractor for images from gelbooru.com based on search-tags"""
-    pattern = BASE_PATTERN + r"page=post&s=list&tags=([^&#]+)"
+    pattern = BASE_PATTERN + r"page=post&s=list&tags=([^&#]*)"
     example = "https://gelbooru.com/index.php?page=post&s=list&tags=TAG"
 
 
@@ -166,13 +196,61 @@ class GelbooruFavoriteExtractor(GelbooruBase,
         params = {
             "s"    : "favorite",
             "id"   : self.favorite_id,
-            "limit": "1",
+            "limit": "2",
         }
+        data = self._api_request(params, None, True)
 
-        count = self._api_request(params, "@attributes")[0]["count"]
-        if count <= self.offset:
-            return
+        count = data["@attributes"]["count"]
+        self.log.debug("API reports %s favorite entries", count)
 
+        favs = data["favorite"]
+        try:
+            order = 1 if favs[0]["id"] < favs[1]["id"] else -1
+        except LookupError as exc:
+            self.log.debug(
+                "Error when determining API favorite order (%s: %s)",
+                exc.__class__.__name__, exc)
+            order = -1
+        else:
+            self.log.debug("API yields favorites in %sscending order",
+                           "a" if order > 0 else "de")
+
+        order_favs = self.config("order-posts")
+        if order_favs and order_favs[0] in ("r", "a"):
+            self.log.debug("Returning them in reverse")
+            order = -order
+
+        if order < 0:
+            return self._pagination(params, count)
+        return self._pagination_reverse(params, count)
+
+    def _pagination(self, params, count):
+        if self.offset:
+            pnum, skip = divmod(self.offset, self.per_page)
+        else:
+            pnum = skip = 0
+
+        params["pid"] = pnum
+        params["limit"] = self.per_page
+
+        while True:
+            favs = self._api_request(params, "favorite")
+
+            if not favs:
+                return
+
+            if skip:
+                favs = favs[skip:]
+                skip = 0
+
+            for fav in favs:
+                for post in self._api_request({"id": fav["favorite"]}):
+                    post["date_favorited"] = text.parse_timestamp(fav["added"])
+                    yield post
+
+            params["pid"] += 1
+
+    def _pagination_reverse(self, params, count):
         pnum, last = divmod(count-1, self.per_page)
         if self.offset > last:
             # page number change
@@ -181,7 +259,6 @@ class GelbooruFavoriteExtractor(GelbooruBase,
             pnum -= diff + 1
         skip = self.offset
 
-        # paginate over them in reverse
         params["pid"] = pnum
         params["limit"] = self.per_page
 
@@ -194,7 +271,9 @@ class GelbooruFavoriteExtractor(GelbooruBase,
                 skip = 0
 
             for fav in favs:
-                yield from self._api_request({"id": fav["favorite"]})
+                for post in self._api_request({"id": fav["favorite"]}):
+                    post["date_favorited"] = text.parse_timestamp(fav["added"])
+                    yield post
 
             params["pid"] -= 1
             if params["pid"] < 0:
