@@ -54,26 +54,21 @@ class KemonopartyExtractor(Extractor):
             sort_keys=True, separators=(",", ":")).encode
 
     def items(self):
-        service = self.groups[2]
-        creator_id = self.groups[3]
-
         find_hash = re.compile(HASH_PATTERN).match
         generators = self._build_file_generators(self.config("files"))
         announcements = True if self.config("announcements") else None
+        archives = True if self.config("archives") else False
         comments = True if self.config("comments") else False
         duplicates = True if self.config("duplicates") else False
         dms = True if self.config("dms") else None
-        profile = username = None
+        max_posts = self.config("max-posts")
+        creator_info = {} if self.config("metadata", True) else None
+        exts_archive = {"zip", "rar", "7z"}
 
         # prevent files from being sent with gzip compression
         headers = {"Accept-Encoding": "identity"}
 
-        if self.config("metadata"):
-            profile = self.api.creator_profile(service, creator_id)
-            username = profile["name"]
-
         posts = self.posts()
-        max_posts = self.config("max-posts")
         if max_posts:
             posts = itertools.islice(posts, max_posts)
         if self.revisions:
@@ -85,10 +80,20 @@ class KemonopartyExtractor(Extractor):
             post["_http_headers"] = headers
             post["date"] = self._parse_datetime(
                 post.get("published") or post.get("added") or "")
+            service = post["service"]
+            creator_id = post["user"]
+
+            if creator_info is not None:
+                key = "{}_{}".format(service, creator_id)
+                if key not in creator_info:
+                    creator = creator_info[key] = self.api.creator_profile(
+                        service, creator_id)
+                else:
+                    creator = creator_info[key]
+
+                post["user_profile"] = creator
+                post["username"] = creator["name"]
 
-            if profile is not None:
-                post["username"] = username
-                post["user_profile"] = profile
             if comments:
                 try:
                     post["comments"] = self.api.creator_post_comments(
@@ -112,11 +117,15 @@ class KemonopartyExtractor(Extractor):
 
             files = []
             hashes = set()
+            post_archives = post["archives"] = []
 
             for file in itertools.chain.from_iterable(
                     g(post) for g in generators):
                 url = file["path"]
 
+                if "\\" in url:
+                    file["path"] = url = url.replace("\\", "/")
+
                 match = find_hash(url)
                 if match:
                     file["hash"] = hash = match.group(1)
@@ -126,31 +135,45 @@ class KemonopartyExtractor(Extractor):
                             continue
                         hashes.add(hash)
                 else:
-                    file["hash"] = ""
-
-                files.append(file)
-
-            post["count"] = len(files)
-            yield Message.Directory, post
-
-            for post["num"], file in enumerate(files, 1):
-                post["_http_validate"] = None
-                post["hash"] = file["hash"]
-                post["type"] = file["type"]
-                url = file["path"]
-
-                text.nameext_from_url(file.get("name", url), post)
-                ext = text.ext_from_url(url)
-                if not post["extension"]:
-                    post["extension"] = ext
-                elif ext == "txt" and post["extension"] != "txt":
-                    post["_http_validate"] = _validate
+                    file["hash"] = hash = ""
 
                 if url[0] == "/":
                     url = self.root + "/data" + url
                 elif url.startswith(self.root):
                     url = self.root + "/data" + url[20:]
-                yield Message.Url, url, post
+                file["url"] = url
+
+                text.nameext_from_url(file.get("name", url), file)
+                ext = text.ext_from_url(url)
+                if not file["extension"]:
+                    file["extension"] = ext
+                elif ext == "txt" and file["extension"] != "txt":
+                    file["_http_validate"] = _validate
+                elif ext in exts_archive:
+                    file["type"] = "archive"
+                    if archives:
+                        try:
+                            data = self.api.posts_archives(file["hash"])
+                            data.update(file)
+                            post_archives.append(data)
+                        except Exception as exc:
+                            self.log.warning(
+                                "%s: Failed to retrieve archive metadata of "
+                                "'%s' (%s: %s)", post["id"], file.get("name"),
+                                exc.__class__.__name__, exc)
+                            post_archives.append(file.copy())
+                    else:
+                        post_archives.append(file.copy())
+
+                files.append(file)
+
+            post["count"] = len(files)
+            yield Message.Directory, post
+            for post["num"], file in enumerate(files, 1):
+                if "id" in file:
+                    del file["id"]
+                post.update(file)
+                yield Message.Url, file["url"], post
 
     def login(self):
         username, password = self._get_auth_info()
@@ -171,7 +194,7 @@ class KemonopartyExtractor(Extractor):
             try:
                 msg = '"' + response.json()["error"] + '"'
             except Exception:
-                msg = '"0/1 Username or password is incorrect"'
+                msg = '"Username or password is incorrect"'
             raise exception.AuthenticationError(msg)
 
         return {c.name: c.value for c in response.cookies}
@@ -296,8 +319,12 @@ class KemonopartyUserExtractor(KemonopartyExtractor):
     def posts(self):
         _, _, service, creator_id, query = self.groups
         params = text.parse_query(query)
-        return self.api.creator_posts(
-            service, creator_id, params.get("o"), params.get("q"))
+        if params.get("tag"):
+            return self.api.creator_tagged_posts(
+                service, creator_id, params.get("tag"), params.get("o"))
+        else:
+            return self.api.creator_posts(
+                service, creator_id, params.get("o"), params.get("q"))
 
 
 class KemonopartyPostsExtractor(KemonopartyExtractor):
@@ -361,17 +388,18 @@ class KemonopartyDiscordExtractor(KemonopartyExtractor):
                 key = "id"
             else:
                 key = "name"
+        else:
+            key = "id"
+            channel = channel_id
 
+        if not channel_name or not channel_id:
             for ch in self.api.discord_server(server_id):
                 if ch[key] == channel:
                     break
             else:
                 raise exception.NotFoundError("channel")
-
             channel_id = ch["id"]
             channel_name = ch["name"]
-        elif channel_name is None:
-            channel_name = ""
 
         find_inline = re.compile(
             r"https?://(?:cdn\.discordapp.com|media\.discordapp\.net)"
@@ -455,9 +483,15 @@ class KemonopartyFavoriteExtractor(KemonopartyExtractor):
                        reverse=(order == "desc"))
 
             for user in users:
-                user["_extractor"] = KemonopartyUserExtractor
-                url = "{}/{}/user/{}".format(
-                    self.root, user["service"], user["id"])
+                service = user["service"]
+                if service == "discord":
+                    user["_extractor"] = KemonopartyDiscordServerExtractor
+                    url = "{}/discord/server/{}".format(
+                        self.root, user["id"])
+                else:
+                    user["_extractor"] = KemonopartyUserExtractor
+                    url = "{}/{}/user/{}".format(
+                        self.root, service, user["id"])
                 yield Message.Queue, url, user
 
         elif type == "post":
@@ -487,14 +521,23 @@ class KemonoAPI():
 
     def posts(self, offset=0, query=None, tags=None):
         endpoint = "/posts"
-        params = {"q": query, "o": offset, "tags": tags}
+        params = {"q": query, "o": offset, "tag": tags}
         return self._pagination(endpoint, params, 50, "posts")
 
+    def posts_archives(self, file_hash):
+        endpoint = "/posts/archives/" + file_hash
+        return self._call(endpoint)["archive"]
+
     def creator_posts(self, service, creator_id, offset=0, query=None):
         endpoint = "/{}/user/{}".format(service, creator_id)
         params = {"q": query, "o": offset}
         return self._pagination(endpoint, params, 50)
 
+    def creator_tagged_posts(self, service, creator_id, tags, offset=0):
+        endpoint = "/{}/user/{}/posts-legacy".format(service, creator_id)
+        params = {"o": offset, "tag": tags}
+        return self._pagination(endpoint, params, 50, "results")
+
     def creator_announcements(self, service, creator_id):
         endpoint = "/{}/user/{}/announcements".format(service, creator_id)
         return self._call(endpoint)
@@ -559,9 +602,10 @@ class KemonoAPI():
             data = self._call(endpoint, params)
 
             if key:
-                yield from data[key]
-            else:
-                yield from data
+                data = data.get(key)
+            if not data:
+                return
+            yield from data
 
             if len(data) < batch:
                 return
