@@ -27,7 +27,7 @@ class PatreonExtractor(Extractor):
     _warning = True
 
     def _init(self):
-        if not self.cookies_check(("session_id",)):
+        if not self.cookies_check(("session_id",), subdomains=True):
             if self._warning:
                 PatreonExtractor._warning = False
                 self.log.warning("no 'session_id' cookie set")
@@ -169,6 +169,12 @@ class PatreonExtractor(Extractor):
             attr["date"] = text.parse_datetime(
                 attr["published_at"], "%Y-%m-%dT%H:%M:%S.%f%z")
 
+            try:
+                attr["campaign"] = (included["campaign"][
+                                    relationships["campaign"]["data"]["id"]])
+            except Exception:
+                attr["campaign"] = None
+
             tags = relationships.get("user_defined_tags")
             attr["tags"] = [
                 tag["id"].replace("user_defined;", "")
@@ -286,15 +292,12 @@ class PatreonExtractor(Extractor):
         return [genmap[ft] for ft in filetypes]
 
     def _extract_bootstrap(self, page):
-        data = text.extr(
-            page, 'id="__NEXT_DATA__" type="application/json">', '</script')
-        if data:
-            try:
-                data = util.json_loads(data)
-                env = data["props"]["pageProps"]["bootstrapEnvelope"]
-                return env.get("pageBootstrap") or env["bootstrap"]
-            except Exception as exc:
-                self.log.debug("%s: %s", exc.__class__.__name__, exc)
+        try:
+            data = self._extract_nextdata(page)
+            env = data["props"]["pageProps"]["bootstrapEnvelope"]
+            return env.get("pageBootstrap") or env["bootstrap"]
+        except Exception as exc:
+            self.log.debug("%s: %s", exc.__class__.__name__, exc)
 
         bootstrap = text.extr(
             page, 'window.patreon = {"bootstrap":', '},"apiServer"')
@@ -326,9 +329,11 @@ class PatreonCreatorExtractor(PatreonExtractor):
     """Extractor for a creator's works"""
     subcategory = "creator"
     pattern = (r"(?:https?://)?(?:www\.)?patreon\.com"
-               r"/(?!(?:home|join|posts|login|signup)(?:$|[/?#]))"
-               r"(?:c/)?([^/?#]+)(?:/posts)?/?(?:\?([^#]+))?")
-    example = "https://www.patreon.com/USER"
+               r"/(?!(?:home|create|login|signup|search|posts|messages)"
+               r"(?:$|[/?#]))"
+               r"(?:profile/creators|(?:c/)?([^/?#]+)(?:/posts)?)"
+               r"/?(?:\?([^#]+))?")
+    example = "https://www.patreon.com/c/USER"
 
     def posts(self):
         creator, query = self.groups
@@ -348,7 +353,7 @@ class PatreonCreatorExtractor(PatreonExtractor):
         return self._pagination(url)
 
     def _get_campaign_id(self, creator, query):
-        if creator.startswith("id:"):
+        if creator and creator.startswith("id:"):
             return creator[3:]
 
         campaign_id = query.get("c") or query.get("campaign_id")
@@ -366,7 +371,7 @@ class PatreonCreatorExtractor(PatreonExtractor):
             data = None
             data = self._extract_bootstrap(page)
             return data["campaign"]["data"]["id"]
-        except (KeyError, ValueError) as exc:
+        except Exception as exc:
             if data:
                 self.log.debug(data)
             raise exception.StopExtraction(
